@@ -5,6 +5,41 @@ import math
 import operator as op
 from functools import reduce
 from .pycm_interpret import *
+from .pycm_ci import kappa_SE_calc, CI_calc, SE_calc
+from .pycm_util import complement
+
+
+def ARI_calc(classes, table, TOP, P, POP):
+    """
+    Calculate ARI (Adjusted Rand index).
+
+    :param classes: classes
+    :type classes : list
+    :param table: input matrix
+    :type table : dict
+    :param TOP: test outcome positive
+    :type TOP : dict
+    :param P: condition positive
+    :type P : dict
+    :param POP: population
+    :type POP : int
+    :return: ARI as float
+    """
+    try:
+        table_sum = 0
+        TOP_sum = 0
+        P_sum = 0
+        nc2 = ncr(POP, 2)
+        for i in classes:
+            TOP_sum += ncr(TOP[i], 2)
+            P_sum += ncr(P[i], 2)
+            for j in classes:
+                table_sum += ncr(table[i][j], 2)
+        x = (TOP_sum * P_sum) / nc2
+        ARI = (table_sum - x) / ((P_sum + TOP_sum) / 2 - x)
+        return ARI
+    except Exception:
+        return "None"
 
 
 def pearson_C_calc(chi_square, POP):
@@ -206,6 +241,8 @@ def ncr(n, r):
     :type r :int
     :return: n choose r as int
     """
+    if r > n:
+        return 0
     r = min(r, n - r)
     numer = reduce(op.mul, range(n, n - r, -1), 1)
     denom = reduce(op.mul, range(1, r + 1), 1)
@@ -364,7 +401,7 @@ def joint_entropy_calc(classes, table, POP):
     try:
         result = 0
         for i in classes:
-            for index, j in enumerate(classes):
+            for j in classes:
                 p_prime = table[i][j] / POP[i]
                 if p_prime != 0:
                     result += p_prime * math.log(p_prime, 2)
@@ -391,7 +428,7 @@ def conditional_entropy_calc(classes, table, P, POP):
         result = 0
         for i in classes:
             temp = 0
-            for index, j in enumerate(classes):
+            for j in classes:
                 p_prime = 0
                 if P[i] != 0:
                     p_prime = table[i][j] / P[i]
@@ -517,7 +554,7 @@ def chi_square_calc(classes, table, TOP, P, POP):
     try:
         result = 0
         for i in classes:
-            for index, j in enumerate(classes):
+            for j in classes:
                 expected = (TOP[j] * P[i]) / (POP[i])
                 result += ((table[i][j] - expected)**2) / expected
         return result
@@ -588,77 +625,20 @@ def reliability_calc(RACC, ACC):
         return "None"
 
 
-def kappa_se_calc(PA, PE, POP):
+def micro_calc(item1, item2):
     """
-    Calculate kappa standard error.
+    Calculate PPV,TPR,TNR,FNR,FPR,F1 micro.
 
-    :param PA: observed agreement among raters (overall accuracy)
-    :type PA : float
-    :param PE:  hypothetical probability of chance agreement (random accuracy)
-    :type PE : float
-    :param POP: population
-    :type POP:int
-    :return: kappa standard error as float
+    :param item1: item1 in micro averaging
+    :type item1:dict
+    :param item2: item2 in micro averaging
+    :type item2 : dict
+    :return: PPV,TPR,TNR,FNR,FPR,F1 micro as float
     """
     try:
-        result = math.sqrt((PA * (1 - PA)) / (POP * ((1 - PE)**2)))
-        return result
-    except Exception:
-        return "None"
-
-
-def CI_calc(mean, SE, CV=1.96):
-    """
-    Calculate confidence interval.
-
-    :param mean: mean of data
-    :type mean : float
-    :param SE: standard error of data
-    :type SE : float
-    :param CV: critical value
-    :type CV:float
-    :return: confidence interval as tuple
-    """
-    try:
-        CI_down = mean - CV * SE
-        CI_up = mean + CV * SE
-        return (CI_down, CI_up)
-    except Exception:
-        return ("None", "None")
-
-
-def se_calc(overall_accuracy, POP):
-    """
-    Calculate standard error with binomial distribution.
-
-    :param overall_accuracy: overall accuracy
-    :type  overall_accuracy : float
-    :type PE : float
-    :param POP: population
-    :type POP : int
-    :return: standard error as float
-    """
-    try:
-        return math.sqrt(
-            (overall_accuracy * (1 - overall_accuracy)) / POP)
-    except Exception:
-        return "None"
-
-
-def micro_calc(TP, item):
-    """
-    Calculate PPV_Micro and TPR_Micro.
-
-    :param TP: true positive
-    :type TP:dict
-    :param item: FN or FP
-    :type item : dict
-    :return: PPV_Micro or TPR_Micro as float
-    """
-    try:
-        TP_sum = sum(TP.values())
-        item_sum = sum(item.values())
-        return TP_sum / (TP_sum + item_sum)
+        item1_sum = sum(item1.values())
+        item2_sum = sum(item2.values())
+        return item1_sum / (item1_sum + item2_sum)
     except Exception:
         return "None"
 
@@ -679,27 +659,6 @@ def macro_calc(item):
         return "None"
 
 
-def PC_PI_calc(P, TOP, POP):
-    """
-    Calculate percent chance agreement for Scott's Pi.
-
-    :param P: condition positive
-    :type P : dict
-    :param TOP: test outcome positive
-    :type TOP : dict
-    :param POP: population
-    :type POP:dict
-    :return: percent chance agreement as float
-    """
-    try:
-        result = 0
-        for i in P.keys():
-            result += ((P[i] + TOP[i]) / (2 * POP[i]))**2
-        return result
-    except Exception:
-        return "None"
-
-
 def PC_AC1_calc(P, TOP, POP):
     """
     Calculate percent chance agreement for Gwet's AC1.
@@ -785,79 +744,40 @@ def overall_random_accuracy_calc(item):
         return "None"
 
 
-def overall_statistics(
-        RACC,
-        RACCU,
-        TPR,
-        PPV,
-        F1,
-        TP,
-        FN,
-        ACC,
-        POP,
-        P,
-        TOP,
-        jaccard_list,
-        CEN_dict,
-        MCEN_dict,
-        AUC_dict,
-        classes,
-        table):
+def overall_statistics(**kwargs):
     """
     Return overall statistics.
 
-    :param RACC: random accuracy
-    :type RACC : dict
-    :param TPR: sensitivity, recall, hit rate, or true positive rate
-    :type TPR : dict
-    :param PPV: precision or positive predictive value
-    :type PPV : dict
-    :param F1: F1 score
-    :type F1: dict
-    :param TP: true positive
-    :type TP : dict
-    :param FN: false negative
-    :type FN : dict
-    :param ACC: accuracy
-    :type ACC: dict
-    :param POP: population
-    :type POP:dict
-    :param P: condition positive
-    :type P : dict
-    :param POP: population
-    :type POP:dict
-    :param TOP: test outcome positive
-    :type TOP : dict
-    :param jaccard_list : list of jaccard index for each class
-    :type jaccard_list : list
-    :param CEN_dict: CEN dictionary for each class
-    :type CEN_dict : dict
-    :param classes: confusion matrix classes
-    :type classes : list
-    :param table: input matrix
-    :type table : dict
+    :param kwargs: inputs
+    :type kwargs: dict
     :return: overall statistics as dict
     """
+    POP = kwargs["POP"]
     population = list(POP.values())[0]
+    TP = kwargs["TP"]
+    P = kwargs["P"]
+    TOP = kwargs["TOP"]
+    table = kwargs["table"]
+    classes = kwargs["classes"]
     overall_accuracy = overall_accuracy_calc(TP, population)
-    overall_random_accuracy_unbiased = overall_random_accuracy_calc(RACCU)
-    overall_random_accuracy = overall_random_accuracy_calc(RACC)
+    overall_random_accuracy_unbiased = overall_random_accuracy_calc(
+        kwargs["RACCU"])
+    overall_random_accuracy = overall_random_accuracy_calc(kwargs["RACC"])
     overall_kappa = reliability_calc(overall_random_accuracy, overall_accuracy)
-    PC_PI = PC_PI_calc(P, TOP, POP)
     PC_AC1 = PC_AC1_calc(P, TOP, POP)
     PC_S = PC_S_calc(classes)
-    PI = reliability_calc(PC_PI, overall_accuracy)
     AC1 = reliability_calc(PC_AC1, overall_accuracy)
     S = reliability_calc(PC_S, overall_accuracy)
-    kappa_SE = kappa_se_calc(
+    kappa_SE = kappa_SE_calc(
         overall_accuracy,
         overall_random_accuracy, population)
     kappa_unbiased = reliability_calc(
         overall_random_accuracy_unbiased,
         overall_accuracy)
+    PI = kappa_unbiased
     kappa_no_prevalence = kappa_no_prevalence_calc(overall_accuracy)
     kappa_CI = CI_calc(overall_kappa, kappa_SE)
-    overall_accuracy_se = se_calc(overall_accuracy, population)
+    overall_accuracy_se = SE_calc(overall_accuracy, population)
     overall_accuracy_CI = CI_calc(overall_accuracy, overall_accuracy_se)
     chi_squared = chi_square_calc(classes, table, TOP, P, POP)
     phi_squared = phi_square_calc(chi_squared, population)
@@ -874,21 +794,27 @@ def overall_statistics(
     lambda_A = lambda_A_calc(classes, table, P, population)
     DF = DF_calc(classes)
     overall_jaccard_index = overall_jaccard_index_calc(list(
-        jaccard_list.values()))
+        kwargs["jaccard_list"].values()))
     hamming_loss = hamming_calc(TP, population)
     zero_one_loss = zero_one_loss_calc(TP, population)
     NIR = NIR_calc(P, population)
     p_value = p_value_calc(TP, population, NIR)
-    overall_CEN = overall_CEN_calc(classes, TP, TOP, P, CEN_dict)
-    overall_MCEN = overall_CEN_calc(classes, TP, TOP, P, MCEN_dict, True)
+    overall_CEN = overall_CEN_calc(classes, TP, TOP, P, kwargs["CEN_dict"])
+    overall_MCEN = overall_CEN_calc(
+        classes, TP, TOP, P, kwargs["MCEN_dict"], True)
     overall_MCC = overall_MCC_calc(classes, table, TOP, P)
     RR = RR_calc(classes, TOP)
     CBA = CBA_calc(classes, table, TOP, P)
-    AUNU = macro_calc(AUC_dict)
-    AUNP = AUNP_calc(classes, P, POP, AUC_dict)
+    AUNU = macro_calc(kwargs["AUC_dict"])
+    AUNP = AUNP_calc(classes, P, POP, kwargs["AUC_dict"])
     RCI = RCI_calc(mutual_information, reference_entropy)
     C = pearson_C_calc(chi_squared, population)
-    TPR_PPV_F1_micro = micro_calc(TP=TP, item=FN)
+    TPR_PPV_F1_micro = micro_calc(item1=TP, item2=kwargs["FN"])
+    TPR_macro = macro_calc(kwargs["TPR"])
+    CSI = macro_calc(kwargs["ICSI_dict"])
+    ARI = ARI_calc(classes, table, TOP, P, population)
+    TNR_micro = micro_calc(item1=kwargs["TN"], item2=kwargs["FP"])
+    TNR_macro = macro_calc(kwargs["TNR"])
     return {
         "Overall ACC": overall_accuracy,
         "Kappa": overall_kappa,
@@ -899,11 +825,17 @@ def overall_statistics(
         "SOA4(Cicchetti)": kappa_analysis_cicchetti(overall_kappa),
         "SOA5(Cramer)": V_analysis(cramer_V),
         "SOA6(Matthews)": MCC_analysis(overall_MCC),
-        "TPR Macro": macro_calc(TPR),
-        "PPV Macro": macro_calc(PPV),
-        "ACC Macro": macro_calc(ACC),
-        "F1 Macro": macro_calc(F1),
+        "TNR Macro": TNR_macro,
+        "TPR Macro": TPR_macro,
+        "FPR Macro": complement(TNR_macro),
+        "FNR Macro": complement(TPR_macro),
+        "PPV Macro": macro_calc(kwargs["PPV"]),
+        "ACC Macro": macro_calc(kwargs["ACC"]),
+        "F1 Macro": macro_calc(kwargs["F1"]),
+        "TNR Micro": TNR_micro,
+        "FPR Micro": complement(TNR_micro),
         "TPR Micro": TPR_PPV_F1_micro,
+        "FNR Micro": complement(TPR_PPV_F1_micro),
         "PPV Micro": TPR_PPV_F1_micro,
         "F1 Micro": TPR_PPV_F1_micro,
         "Scott PI": PI,
@@ -942,4 +874,6 @@ def overall_statistics(
         "AUNU": AUNU,
         "AUNP": AUNP,
         "RCI": RCI,
-        "Pearson C": C}
+        "Pearson C": C,
+        "CSI": CSI,
+        "ARI": ARI}
