@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 """Parameters and constants."""
-VERSION = "2.3"
+PYCM_VERSION = "2.6"
 
 
 OVERVIEW = '''
@@ -12,7 +12,7 @@ PyCM is the swiss-army knife of confusion matrices, targeted mainly at
 data scientists that need a broad array of metrics for predictive models
 and an accurate evaluation of large variety of classifiers.
 
-If you use PyCM in your research, please cite this paper :
+If you use PyCM in your research, we would appreciate citations to the following paper :
 
 https://doi.org/10.21105/joss.00729
 
@@ -28,6 +28,7 @@ VECTOR_SIZE_ERROR = "Input vectors must have same length"
 VECTOR_EMPTY_ERROR = "Input vectors are empty"
 CLASS_NUMBER_ERROR = "Number of the classes is lower than 2"
 COMPARE_FORMAT_ERROR = "The input type is considered to be dictionary but it's not!"
+
 COMPARE_TYPE_ERROR = "The input is considered to consist of pycm.ConfusionMatrix object but it's not!"
 COMPARE_DOMAIN_ERROR = "The domain of all ConfusionMatrix objects must be same! The sample size or the number " \
                        "of classes are different."
@@ -39,15 +40,88 @@ COMPARE_WEIGHT_ERROR = "The weight type must be dictionary and also must be set
 COMPARE_RESULT_WARNING = "Confusion matrices are too close and the best one can not be recognized."
 
 CLASS_NUMBER_WARNING = "The confusion matrix is a high dimension matrix and won't be demonstrated properly.\n" \
-                       "The save_csv method can be used to save the confusion matrix in csv format and have a better" \
-                       " demonstration of it."
+                       "If confusion matrix has too many zeros (sparse matrix) you can set `sparse` flag to True in printing functions "\
+                       "otherwise by using save_csv method to save the confusion matrix in csv format you'll have better demonstration."
 
 CLASS_NUMBER_THRESHOLD = 10
 
 BALANCE_RATIO_THRESHOLD = 3
 
+SUMMARY_OVERALL = [
+    "ACC Macro",
+    "Kappa",
+    "Overall ACC",
+    "SOA1(Landis & Koch)",
+    "Zero-one Loss",
+    "F1 Macro",
+    "TPR Macro",
+    "PPV Macro",
+    "FPR Macro"]
+
+SUMMARY_CLASS = [
+    "ACC",
+    "AUC",
+    "AUCI",
+    "F1",
+    "TPR",
+    "FPR",
+    "PPV",
+    "TP",
+    "FP",
+    "FN",
+    "TN",
+    "N",
+    "P",
+    "POP",
+    "TOP",
+    "TON"]
+
 BINARY_RECOMMEND = ["ACC", "TPR", "PPV", "AUC", "AUCI", "TNR", "F1"]
 
+CI_CLASS_LIST = [
+    "TPR",
+    "TNR",
+    "PPV",
+    "NPV",
+    "ACC",
+    "PLR",
+    "NLR",
+    "FPR",
+    "FNR",
+    "AUC",
+    "PRE"]
+
+CI_OVERALL_LIST = ["Kappa", "Overall ACC"]
+
+ALPHA_TWO_SIDE_TABLE = {
+    0.2: 1.28,
+    0.1: 1.645,
+    0.05: 1.96,
+    0.02: 2.326,
+    0.01: 2.576,
+    0.002: 3.09,
+    0.001: 3.29}
+
+ALPHA_ONE_SIDE_TABLE = {
+    0.1: 1.28,
+    0.05: 1.645,
+    0.01: 2.326,
+    0.005: 2.576,
+    0.001: 3.09,
+    0.0005: 3.29}
+
+CI_ALPHA_TWO_SIDE_WARNING = "The alpha value is invalid, automatically set to 0.05.\nSupported values (two-sided) : " + ",".join(
+    map(str, sorted(list(ALPHA_TWO_SIDE_TABLE.keys()))))
+
+CI_ALPHA_ONE_SIDE_WARNING = "The alpha value is invalid, automatically set to 0.05.\nSupported values (one-sided) : " + ",".join(
+    map(str, sorted(list(ALPHA_ONE_SIDE_TABLE.keys()))))
+
+CI_FORMAT_ERROR = "The input type is considered to be string but it's not!"
+
+CI_SUPPORT_ERROR = "CI calculation for this parameter is not supported on this version of pycm.\nSupported parameters : " + \
+    ",".join(CI_CLASS_LIST) + "," + ",".join(CI_OVERALL_LIST)
+
+
 MULTICLASS_RECOMMEND = [
     "ERR",
     "TPR Micro",
@@ -93,6 +167,13 @@ NLRI_SCORE = {"Good": 4, "Fair": 3, "Poor": 2, "Negligible": 1, "None": "None"}
 
 DPI_SCORE = {"Good": 4, "Fair": 3, "Limited": 2, "Poor": 1, "None": "None"}
 
+QI_SCORE = {
+    "Strong": 4,
+    "Moderate": 3,
+    "Weak": 2,
+    "Negligible": 1,
+    "None": "None"}
+
 AUCI_SCORE = {
     "Excellent": 5,
     "Very Good": 4,
@@ -149,7 +230,8 @@ CLASS_BENCHMARK_SCORE_DICT = {
     "NLRI": NLRI_SCORE,
     "DPI": DPI_SCORE,
     "AUCI": AUCI_SCORE,
-    "MCCI": SOA6_SCORE}
+    "MCCI": SOA6_SCORE,
+    "QI": QI_SCORE}
 
 OVERALL_BENCHMARK_SCORE_DICT = {
     "SOA1(Landis & Koch)": SOA1_SCORE,
@@ -170,6 +252,7 @@ RECOMMEND_HTML_MESSAGE2 = '<span style="color:red;">Note 2</span> : {0}'.format(
     RECOMMEND_WARNING)
 
 DOCUMENT_ADR = "http://www.pycm.ir/doc/index.html#"
+DOCUMENT_ADR_ALT = "https://nbviewer.jupyter.org/github/sepandhaghighi/pycm/blob/master/Document/Document.ipynb#"
 
 PARAMS_DESCRIPTION = {
     "TPR": "sensitivity, recall, hit rate, or true positive rate",
@@ -227,10 +310,13 @@ PARAMS_DESCRIPTION = {
     "IBA": "Index of balanced accuracy",
     "GM": "G-mean geometric mean of specificity and sensitivity",
     "Q": "Yule Q - coefficient of colligation",
+    "QI": "Yule Q interpretation",
     "AGM": "Adjusted geometric mean",
     "AGF": "Adjusted F-score",
     "OC": "Overlap coefficient",
-    "OOC": "Otsuka-Ochiai coefficient"}
+    "OOC": "Otsuka-Ochiai coefficient",
+    "AUPR": "Area under the PR curve",
+    "ICSI": "Individual classification success index"}
 
 PARAMS_LINK = {
     "TPR": "TPR-(True-positive-rate)",
@@ -273,11 +359,17 @@ PARAMS_LINK = {
     "SOA3(Altman)": "SOA3-(Altman's-benchmark)",
     "SOA4(Cicchetti)": "SOA4-(Cicchetti's-benchmark)",
     "TPR Macro": "TPR_Macro",
+    "FNR Macro": "FNR_Macro",
+    "TNR Macro": "TNR_Macro",
+    "FPR Macro": "FPR_Macro",
     "PPV Macro": "PPV_Macro",
     "F1 Macro": "F1_Macro",
     "F1 Micro": "F1_Micro",
     "ACC Macro": "ACC_Macro",
     "TPR Micro": "TPR_Micro",
+    "FNR Micro": "FNR_Micro",
+    "TNR Micro": "TNR_Micro",
+    "FPR Micro": "FPR_Micro",
     "PPV Micro": "PPV_Micro",
     "Scott PI": "Scott's-Pi",
     "Gwet AC1": "Gwet's-AC1",
@@ -295,7 +387,7 @@ PARAMS_LINK = {
     "Cross Entropy": "Cross-entropy",
     "Joint Entropy": "Joint-entropy",
     "Conditional Entropy": "Conditional-entropy",
-    "KL Divergence": "Kullback-Liebler-divergence",
+    "KL Divergence": "Kullback-Leibler-divergence",
     "Lambda B": "Goodman-&-Kruskal's-lambda-B",
     "Lambda A": "Goodman-&-Kruskal's-lambda-A",
     "Kappa Unbiased": "Kappa-unbiased",
@@ -343,9 +435,14 @@ PARAMS_LINK = {
     "SOA6(Matthews)": "SOA6-(Matthews's-benchmark)",
     "AGF": "AGF-(Adjusted-F-score)",
     "OC": "OC-(Overlap-coefficient)",
-    "OOC": "OOC-(Otsuka-Ochiai-coefficient)"}
+    "OOC": "OOC-(Otsuka-Ochiai-coefficient)",
+    "AUPR": "AUPR-(Area-under-the-PR-curve)",
+    "ICSI": "ICSI-(Individual-classification-success-index)",
+    "CSI": "CSI-(Classification-success-index)",
+    "QI": "QI-(Yule's-Q-interpretation)",
+    "ARI": "ARI-(Adjusted-Rand-index)"}
 
-CAPITALIZE_FILTER = ["BCD", "AUCI", "Q", "AGF", "OOC"]
+CAPITALIZE_FILTER = ["BCD", "AUCI", "Q", "AGF", "OOC", "AUPR", "AUC", "QI"]
 
 BENCHMARK_COLOR = {
     "PLRI": {
@@ -354,6 +451,12 @@ BENCHMARK_COLOR = {
         "Fair": "Yellow",
         "Good": "Green",
         "None": "White"},
+    "QI": {
+        "Negligible": "Red",
+        "Weak": "Orange",
+        "Moderate": "Yellow",
+        "Strong": "Green",
+        "None": "White"},
     "NLRI": {
         "Negligible": "Red",
         "Poor": "Orange",
